# LOAD httpfs;
# SET s3_region='us-east-1';
# SET s3_url_style='path';
# SET s3_endpoint='192.168.18.206:9000';
# SET s3_access_key_id='iamdev';
# SET s3_secret_access_key='xzd19950506';
# SET s3_use_ssl=false;

from dadude.reader.duck_client import Quack


if __name__ == "__main__":
    quack = Quack()
    quack.client.execute("""
        LOAD httpfs;
        INSTALL delta;
        LOAD delta;
    """)
    quack.client.execute("""CREATE SECRET my_secret (
    TYPE S3,
    KEY_ID 'iamdev',
    SECRET 'xzd19950506',
    REGION '',
    USE_SSL false,
    URL_STYLE 'path',
    ENDPOINT '192.168.18.206:9000'
);""")
    delta_table_path = "s3a://test/property"
    print(quack._sql(
        "select * from read_parquet('s3a://test/property/0-733054a2-a87a-4a26-91f7-aa86a2f501cd-0.parquet')"
    ))
    dump = quack._sql(f"select * from delta_scan('{delta_table_path}')").to_json()
    print(dump)



import pytest
import pandas as pd
from dadude.reader.delta_client import read_delta_table


@pytest.fixture
def qa_table_path():
    return "s3://test/property"

def test_read_delta_table(qa_table_path):
    df = read_delta_table(qa_table_path)
    assert isinstance(df, pd.DataFrame)
    assert set(df.columns) == {
        "ppi_id",
        "class_ii_name",
        "print_friendly_units",
        "statistical_type",
        "upper_bound",
        "class_i_name",
        "lower_bound",
        "created_at",
        "display_name_en",
        "application_domain",
        "domain_id",
        "domain_name_cn",
        "parent_ppi_id",
        "alias_list_en",
        "class_iii_name",
        "updated_at",
        "display_name_cn",
        "unit_recognition",
        "alias_list_cn",
    }
    assert len(df) == 1383



import pytest
from dadude.primitives import ObjectStorageClient


@pytest.fixture
def self_hosted_minio_client():
    minio_config = {
        "access_key": "iamdev",
        "secret_key": "xzd19950506",
        "endpoint_url": "https://s3.completenonsense.net",
    }
    my_client = ObjectStorageClient.from_config(minio_config)
    return my_client


@pytest.fixture
def patsnap_ceph_client():
    patsnap_ceph_config = {
        "access_key": "k2W8hPYxUP5NTbFB",
        "secret_key": "icrIfxbLNv8gtURdRkFJH3lbIWMSixDT",
        "endpoint_url": "http://local-minio-s3-api.patsnap.io",
    }
    client = ObjectStorageClient.from_config(patsnap_ceph_config)
    return client


@pytest.fixture
def materials_bucket_prefix():
    return "local-rd-common/matter-most"


def test_ceph_client_list_bucket(patsnap_ceph_client):
    buckets = patsnap_ceph_client.list_buckets()["Buckets"]
    assert len(buckets) == 4
    bucket_names = [b["Name"] for b in buckets]
    # if this fails, new buckets have been added to the minio instance
    assert bucket_names == [
        "llm-model",
        "local-llm",
        "local-rd-common",
        "local-rd-datasets",
    ]


def test_ceph_client_list_resources(patsnap_ceph_client, materials_bucket_prefix):
    bucket, prefix = materials_bucket_prefix.split("/", 1)
    all_objects = patsnap_ceph_client.list_objects_v2(
        Bucket=bucket,
        Delimiter="/",
        Prefix=prefix,
    )
    print(all_objects)


import os
from dotenv import load_dotenv
from dataclasses import dataclass
import boto3


load_dotenv()


@dataclass
class ObjectStorageClient:
    access_key: str
    secret_key: str
    endpoint_url: str | None = None
    use_proxy: bool = False

    def __post_init__(self):
        self.client = boto3.client(
            "s3",
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key,
            endpoint_url=self.endpoint_url,
        )

    @classmethod
    def from_config(cls, dict_config: dict):
        if "use_proxy" not in dict_config:
            dict_config["use_proxy"] = False
        return cls(
            dict_config["access_key"],
            dict_config["secret_key"],
            dict_config["endpoint_url"],
            dict_config["use_proxy"],
        ).client

    @classmethod
    def from_env(cls):
        return cls(
            os.getenv("STORAGE_ACCESS_KEY_ID", ""),
            os.getenv("STORAGE_SECRET_ACCESS_KEY", ""),
            os.getenv("STORAGE_ENDPOINT_URL"),
        ).client

    # Bucket level operations
    def list_buckets(self):
        return self.client.list_buckets()

    def create_bucket(self, bucket: str):
        return self.client.create_bucket(Bucket=bucket)

    def delete_bucket(self, bucket: str):
        return self.client.delete_bucket(Bucket=bucket)

    # Object level operations
    def list_objects_v2(self, **kwargs):
        return self.client.list_objects_v2(**kwargs)

    def get_object(self, **kwargs):
        return self.client.get_object(**kwargs)

    def put_object(self, **kwargs):
        return self.client.put_object(**kwargs)


@dataclass
class S3RemoteObject:
    name: str
    last_modified_date: str
    last_modified_time: str
    size: int
    storage_class: str = "STANDARD"
    prefix: str = ""

    @classmethod
    def from_response(cls, s3_response: dict, prefix):
        return cls(
            s3_response["Key"],
            s3_response["LastModified"].strftime("%Y-%m-%d"),
            s3_response["LastModified"].strftime("%H:%M:%S"),
            s3_response["Size"],
            s3_response["StorageClass"],
            prefix,
        )

    @staticmethod
    def parse_size_unit(size):
        if size < 1024:
            return str(size) + "B"
        elif size < 1024**2:
            return str(size // 1024) + "KB"
        elif size < 1024**3:
            return str(size // 1024**2) + "MB"
        else:
            return str(size // 1024**3) + "GB"

    def __str__(self):
        return "\t".join(
            (
                self.last_modified_date,
                self.last_modified_time,
                self.parse_size_unit(self.size),
                self.name[len(self.prefix) :],
            )
        )


if __name__ == "__main__":
    minio_config = {
        "access_key": "iamdev",
        "secret_key": "xzd19950506",
        # "endpoint_url": "https://s3.completenonsense.net",
        "endpoint_url": "http://192.168.18.206:9000",
    }
    my_client = ObjectStorageClient.from_config(minio_config)
    all_objects = my_client.list_objects_v2(
        Bucket="matter-most",
        Delimiter="/",
        Prefix="bronze/",
    )["Contents"]
    print(all_objects)



from enum import Enum
from loguru import logger

# import pyarrow.dataset as ds
import pandas as pd
from deltalake.data_catalog import DataCatalog
from deltalake import write_deltalake

from ..config import default_storage_config, DeltaStorageTier, DEFAULT_BUCKET


logger = logger.bind(name=__name__)
catalog = DataCatalog.UNITY


class DeltaWriterMode(str, Enum):
    OVERWRITE = "overwrite"
    APPEND = "append"
    IGNORE = "ignore"
    ERROR = "error"


class DeltaWriterSchemaMode(str, Enum):
    MERGE = "merge"
    OVERWRITE = "overwrite"


class DeltaWriterError(Exception):
    pass


class DeltaWriter:
    def __init__(self, bucket: str = DEFAULT_BUCKET) -> None:
        self.catalog = catalog
        self.storage_options = default_storage_config
        self.bucket_path = f"s3a://{bucket}"

    def _write(
        self,
        resource_path: str,
        table: pd.DataFrame,
        mode: DeltaWriterMode = DeltaWriterMode.ERROR,
    ):
        logger.debug(f"Start: writing data to {resource_path}.")
        logger.warning(f"Mode: {mode}.")
        try:
            write_deltalake(
                resource_path,
                table,
                storage_options=self.storage_options,
                mode=mode,  # type: ignore
            )
            logger.info(f"Done: {resource_path=} written.")
        except Exception as e:
            logger.error(f"Error: {e}.")
            raise DeltaWriterError(e)

    def _write_with_schema_mode(
        self,
        resource_path: str,
        table: pd.DataFrame,
        data_mode: DeltaWriterMode,
        schema_mode: DeltaWriterSchemaMode = DeltaWriterSchemaMode.MERGE,
    ):
        # TODO: this should not be a separate function
        logger.warning("Check: entering schema merge mode.")
        logger.debug(f"Start: writing data to {resource_path}.")
        logger.warning(f"Mode: {data_mode=}, {schema_mode=}.")
        try:
            write_deltalake(
                resource_path,
                table,
                storage_options=self.storage_options,
                mode=data_mode,  # type: ignore
                schema_mode=schema_mode,  # type: ignore
                engine="rust",
            )
        except Exception as e:
            logger.error(f"Error: {e}.")
            raise DeltaWriterError(e)

    def write_json(
        self,
        json_file_path: str,
        lines=False,
        mode: DeltaWriterMode = DeltaWriterMode.ERROR,
        schema_mode: DeltaWriterSchemaMode | None = None,
    ):
        tier = DeltaStorageTier(json_file_path.split("/")[-2]).value
        table_name = json_file_path.split("/")[-1].split(".")[0].split("_v")[0]
        lake_path = f"{self.bucket_path}/{tier}/{table_name}"
        # TODO: load json file into pyarrow table with schema inference
        # we use pd.DataFrame for now
        df = pd.read_json(json_file_path, lines=lines)
        logger.info(f"View: {df.head(1)}")
        if schema_mode is not None:
            self._write_with_schema_mode(lake_path, df, mode, schema_mode)
        else:
            self._write(lake_path, df, mode)
        logger.info(f"Done: {table_name=} written to {lake_path=}.")


def write_json_table(
    local_json_file_path: str,
    lines: bool = False,
    mode: DeltaWriterMode = DeltaWriterMode.ERROR,
):
    writer = DeltaWriter()
    writer.write_json(local_json_file_path, lines=lines, mode=mode)


def overwrite_json_table(
    local_json_file_path: str,
    lines: bool = False,
    mode: DeltaWriterMode = DeltaWriterMode.OVERWRITE,
):
    writer = DeltaWriter()
    writer.write_json(
        local_json_file_path,
        lines=lines,
        mode=mode,
        schema_mode=DeltaWriterSchemaMode.OVERWRITE,
    )



import os
from dotenv import load_dotenv
from loguru import logger
import boto3
from dadude.primitives import ObjectStorageClient


def manual():
    load_dotenv()
    access_key = os.getenv("STORAGE_ACCESS_KEY_ID")
    secret_key = os.getenv("STORAGE_SECRET_ACCESS_KEY")
    endpoint = os.getenv("STORAGE_ENDPOINT_URL", "")
    logger.debug(f"{access_key=}, {secret_key=}, {endpoint=}")
    session = boto3.client(
        "s3",
        aws_access_key_id=access_key,
        aws_secret_access_key=secret_key,
        endpoint_url=endpoint,
    )
    bucket_response = session.list_buckets()
    # logger.debug(f"{bucket_response=}")
    print([b["Name"] for b in bucket_response.get("Buckets", [])])


def enclosure():
    conn_client = ObjectStorageClient.from_env()
    print(conn_client.list_buckets())


if __name__ == "__main__":
    # manual()
    enclosure()


import os
from deltalake import DeltaTable, write_deltalake
import pyarrow as pa
from dotenv import load_dotenv


def manual():
    load_dotenv()
    # write some data into a delta table
    df = pa.table({"id": [1, 2], "value": ["foo", "bar"]})
    print(df)

    storage_options = {
        "AWS_ACCESS_KEY_ID": os.getenv("STORAGE_ACCESS_KEY_ID", ""),
        "AWS_SECRET_ACCESS_KEY": os.getenv("STORAGE_SECRET_ACCESS_KEY", ""),
        "AWS_ENDPOINT_URL": os.getenv("STORAGE_ENDPOINT_URL", ""),
        "AWS_REGION": "local",
        "AWS_S3_ALLOW_UNSAFE_RENAME": "true",
        "AWS_ALLOW_HTTP": "true"
    }
    write_deltalake("s3a://test/pa/", df, storage_options=storage_options, mode="overwrite")
    print("\nData written\n")
    # read the data back
    table = DeltaTable("s3a://test/pa/", storage_options=storage_options)
    print(table.to_pyarrow_table())


if __name__ == "__main__":
    manual()



import subprocess
from dadude.primitives import ObjectStorageClient


if __name__ == "__main__":
    conn_client = ObjectStorageClient.from_env()
    bucket_name = "matter-most"
    # collection of directories
    directory_names = [
        "inbox",
        "bronze",
        "silver",
        "gold",
        "evaluation",
        "staging",
    ]
    for dir_name in directory_names:
        conn_client.put_object(Bucket=bucket_name, Key=(dir_name + "/"))
    # check if the folders are created
    subprocess.run(["bash", "/home/kevinxu/hack/min3", "ls", "s3://matter-most/"])


import os
from enum import Enum
from dotenv import load_dotenv


load_dotenv()


DEFAULT_BUCKET = "matter-most"

default_storage_config = {
    "AWS_ACCESS_KEY_ID": os.getenv("STORAGE_ACCESS_KEY_ID", ""),
    "AWS_SECRET_ACCESS_KEY": os.getenv("STORAGE_SECRET_ACCESS_KEY", ""),
    "AWS_ENDPOINT_URL": os.getenv("STORAGE_ENDPOINT_URL", ""),
    "AWS_REGION": "local",
    "AWS_S3_ALLOW_UNSAFE_RENAME": "true",
    "AWS_ALLOW_HTTP": "true",
}


class DeltaStorageTier(str, Enum):
    BRONZE = "bronze"
    SILVER = "silver"
    GOLD = "gold"
    INBOX = "inbox"
    EVALUATION = "evaluation"
    STAGING = "staging"



from fire import Fire
import duckdb


def csv2json(csv_path: str) -> str:
    target_json_path = csv_path.replace(".csv", ".json")
    con = duckdb.connect(database=":memory:")
    con.execute(
        f"copy (SELECT * FROM read_csv('{csv_path}')) to '{target_json_path}'"
    )
    con.close()
    return target_json_path


def csv2parquet(csv_path: str) -> str:
    target_parquet_path = csv_path.replace(".csv", ".parquet")
    con = duckdb.connect(database=":memory:")
    con.execute(
        f"copy (SELECT * FROM read_csv('{csv_path}')) to '{target_parquet_path}'"
    )
    con.close()
    return target_parquet_path


def json2parquet(json_path: str) -> str:
    target_parquet_path = json_path.replace(".json", ".parquet")
    con = duckdb.connect(database=":memory:")
    con.execute(
        f"copy (SELECT * FROM read_json('{json_path}')) to '{target_parquet_path}'"
    )
    con.close()
    return target_parquet_path


if __name__ == "__main__":
    Fire()


from loguru import logger
import pandas as pd
from deltalake import DeltaTable
# from deltalake.data_catalog import DataCatalog

from ..config import default_storage_config, DeltaStorageTier


logger = logger.bind(name=__name__)


class DeltaClientError(Exception):
    """Base class for exceptions in this module."""

    pass


def read_delta_table(
    bucket: str,
    tier: str,
    table: str,
    version: int | None = None,
    preview: bool = False,
    local_cache_dir: str | None = None,
) -> pd.DataFrame:
    tier_path = DeltaStorageTier(tier).value
    table_path = f"s3a://{bucket}/{tier_path}/{table}"
    if version:
        logger.info(f"Reading {table} version {version}")
    else:
        logger.info(f"Reading latest version of {table}")
    try:
        dt = DeltaTable(
            table_path, version=version, storage_options=default_storage_config
        )
    except Exception as e:
        logger.error(f"Error reading DeltaTable: {e}")
        raise DeltaClientError(f"Error reading DeltaTable: {e}")
    df_pdf = dt.to_pandas()
    if preview:
        logger.info(f"Previewing first 5 rows of {table_path}")
        print(df_pdf.head())
    if local_cache_dir:
        logger.info(f"Saving DeltaTable to {local_cache_dir}")
        df_pdf.to_json(
            f"{local_cache_dir}/{table}.json", orient="records", lines=True, index=False
        )
    return df_pdf


def read_delta_table_from_catalog():
    raise NotImplementedError
    # return DeltaTable.from_data_catalog(
    #     data_catalog=DataCatalog.UNITY,
    #     database_name="materials",
    #     table_name="property_entity",
    # )


# TODO: add reading from local cache



from typing import Callable
import duckdb


class Quack:
    """
    Unified Duckdb client. (incomplete)
    For unsafe usage, just use _sql.
    """

    def __init__(self, db_path: str = ":memory:", read_only: bool = False):
        self.db_path = db_path
        self.client = duckdb.connect(database=db_path, read_only=read_only)

    def _sql(self, query, limit=None, filters=None, mode="df"):
        query = query.strip()
        query = query.replace(";", "")
        if filters is not None:
            query += " WHERE " + " AND ".join(filters)
        if limit is not None:
            query += " LIMIT " + str(limit)
        print(query)
        if mode == "df":
            return self.client.execute(query + ";").df()
        else:
            return self.client.execute(query + ";").arrow()

    def create_table(self, table_name, file_path):  # , format="parquet"
        # if file_path.endswith(".csv"):
        #     format = "csv"
        #     read_query = "read_csv"
        # elif file_path.endswith(".json"):
        #     format = "json"
        #     read_query = "read_json"
        # else:
        #     if format == "parquet":
        #         read_query = "read_parquet"

        self._sql(f"CREATE TABLE '{table_name}' AS SELECT * FROM '{file_path}'")

    def create_function(self, func_name: str, func: Callable):
        self.client.create_function(func_name, func)

    def inspect_parquet_metadata(self, file_path):
        return self._sql(f"FROM parquet_metadata('{file_path}')")

    def inspect_parquet_schema(self, file_path):
        return self._sql(f"FROM parquet_schema('{file_path}')")

    def get_table(self, table_name):
        return self.client.table(table_name)

    def to_pandas(self, table_name):
        return self.get_table(table_name).df()

    def to_csv(self, query, target_file_path, header=True, sep=","):
        dump_query = f"COPY ({query}) TO '{target_file_path}' (FORMAT CSV, HEADER={header}, DELIMITER='{sep}');"
        return self._sql(dump_query)

    def to_parquet(self, query, target_file_path):
        dump_query = f"COPY ({query}) TO '{target_file_path}' (FORMAT PARQUET);"
        return self._sql(dump_query)

    def list_tables(self):
        return self._sql("SHOW tables;")

    def schema(self, table):
        return self._sql(f"DESC {table}")

    def describe(self, table):
        return self._sql(f"SUMMARIZE {table}")

    def count(self, table):
        return self._sql(f"SELECT COUNT(*) count_{table} FROM {table};")

    def select(self, table, limit=None, columns="*"):
        if isinstance(columns, list):
            columns = ", ".join(columns)
        return self._sql(f"SELECT {columns} FROM {table}", limit)

    def distinct(self, table, column=None):
        if column is None:
            column = self.schema.iloc[0, 0]
        return self._sql(f"SELECT DISTINCT {column} FROM {table};")

    def head(self, table, top=5):
        return self._sql(f"FROM {table};", top)

    def sample(self, table, n_samples=5):
        return self._sql(f"FROM {table} TABLESAMPLE {n_samples} ROWS;")



import os
import boto3
from fire import Fire
from dotenv import load_dotenv
from decimal import Decimal
import json


class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Decimal):
            return str(o)
        return json.JSONEncoder.default(self, o)


def main():
    load_dotenv()
    table_name = os.environ.get("DB_TABLE_NAME", "")
    target_dir = os.environ.get("TG_DIR", "")
    target_path = table_name + ".json"
    db = boto3.resource(
        "dynamodb",
        aws_access_key_id=os.environ.get("DB_ACCESS_KEY_ID"),
        aws_secret_access_key=os.environ.get("DB_SECRET_ACCESS_KEY"),
        region_name=os.environ.get("DB_REGION"),
        endpoint_url=os.environ.get("DB_ENDPOINT_URL"),
    )
    table = db.Table(table_name) # type: ignore
    response = table.scan()
    to_write = response.get("Items")
    with open(f"{target_dir}/{target_path}", "wt") as f:
        json.dump(to_write, f, indent=4, cls=DecimalEncoder, ensure_ascii=False)


if __name__ == "__main__":
    Fire(main)



from functools import partial
from fire import Fire
from ..writer.delta_writer import write_json_table, overwrite_json_table


write_property_entity_v3 = partial(
    write_json_table,
    local_json_file_path="data/gold/material_property_entity_v3.json",
)

write_cheman_substance_entity = partial(
    write_json_table,
    lines=True,
    local_json_file_path="data/silver/cheman_substance_entity.json",
)

overwrite_cheman_substance_entity = partial(
    write_json_table,
    lines=True,
    local_json_file_path="data/silver/cheman_substance_entity_v2.json",
)

write_dc_annotation_spum_cn_clms_2407 = partial(
    write_json_table,
    lines=True,
    local_json_file_path="data/bronze/dc_annotation_spum_cn_clms_2407.json",
)

overwrite_dc_annotation_spum_cn_clms_2407 = partial(
    overwrite_json_table,
    lines=True,
    local_json_file_path="data/bronze/dc_annotation_spum_cn_clms_2407.json",
)

write_dc_annotation_spum_cn_desc_2407 = partial(
    write_json_table,
    lines=True,
    local_json_file_path="data/bronze/dc_annotation_spum_cn_desc_2407.json",
)

if __name__ == "__main__":
    Fire()



import os
import gzip
from decimal import Decimal
import json
import boto3
from tqdm import tqdm
from fire import Fire
from loguru import logger
from dotenv import load_dotenv


SAMPLE_ROWS = 1000
DECIMAL_PRECISION = 2


def float2decimal(s):
    return Decimal(str(round(float(s), DECIMAL_PRECISION)))


def str2gz(text):
    """
    str to Dynamodb binary
    :param text: text to compress
    :return:
    """
    data = text.encode("utf-8")
    gziped = gzip.compress(data)
    return gziped


def gz2str(data):
    """
    Dynamodb binary to str
    :param data: byte[]
    :return: string
    """
    flat = gzip.decompress(data)
    return flat.decode("utf-8")


def main(
    input_file,
    table_name,
    uid,
    limit=SAMPLE_ROWS,
    lines=True,
):
    load_dotenv()
    db = boto3.resource(
        "dynamodb",
        aws_access_key_id=os.environ.get("QA_DB_ACCESS_KEY_ID"),
        aws_secret_access_key=os.environ.get("DB_SECRET_ACCESS_KEY"),
        region_name=os.environ.get("DB_REGION"),
        endpoint_url=os.environ.get("QA_DB_ENDPOINT_URL"),
    )
    logger.info(f"Connected to {db.meta.client.meta.endpoint_url}")     # type: ignore
    table = db.Table(table_name)  # type: ignore
    logger.info(f"Table: {table.name}")
    # start writing to table
    with open(input_file) as f:
        written = 0
        rows = []
        if lines:
            for line in f:
                rows.append(json.loads(line, parse_float=float2decimal))
        else:
            rows = json.load(f, parse_float=float2decimal)
        for entry in tqdm(rows):
            if written > limit:
                break
            response = table.put_item(Item=entry)
            if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
                written += 1
            else:
                logger.error(f"Failed to write {entry[uid]}")


if __name__ == "__main__":
    Fire(main)
